/* This file is part of fileserv.
   Copyright (C) 2017 Sergey Poznyakoff

   Fileserv is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   Fileserv is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with fileserv.  If not, see <http://www.gnu.org/licenses/>. */
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <stdarg.h>
#include <regex.h>

#ifndef SIZE_T_MAX
# define SIZE_T_MAX ((size_t)-1)
#endif

extern char *progname;
extern char *pidfile;
extern char *index_css;
extern char *tmpdir;

void error(char const *fmt, ...);
void info(char const *fmt, ...);
void syslog_enable(void);
void set_log_facility(char const *arg);

void fileserv_logger(void *arg, const char *fmt, va_list ap);

void xmalloc_fail(void);
void *xmalloc(size_t s);
void *xcalloc(size_t nmemb, size_t size);
void *xrealloc(void *ptr, size_t size);
void *xnrealloc(void *ptr, size_t nmemb, size_t size);
char *xstrdup(char const *str);

void runas(char const *runas_user, char const *runas_group);

#if HAVE_LIBWRAP
int fileserv_acl(void *cls, const struct sockaddr *addr, socklen_t addrlen);
#else
# define fileserv_acl NULL
#endif

void pidfile_remove(void);
void pidfile_create(void);
void pidfile_check(void);

typedef struct fileserv_dirconfig {
	int follow:1;            /* Follow symbolic links */
	int listing:1;           /* Show directory listing, if a directory
				    is requested and no index file exists */
	int list_unreadable:1;   /* List unreadable files */
	char **index_files;      /* Names of index files */
	regex_t *hidden_files_rxv;
	size_t hidden_files_rxc;
} DIRCONFIG;

#define DIRCONFIG_INITIALIZER { 0, 0, 1, NULL }

DIRCONFIG *dirconfig(char const *path, size_t prefix_len);
void dirconfig_parse(const char *file, DIRCONFIG *conf);
DIRCONFIG *dirconfig_init(void);
void dirconfig_free(DIRCONFIG *conf);

char *catfile_n(char const *dir, size_t len, char const *file);
char *catfile(char const *dir, char const *file);

typedef enum {
	ISC_NAME,
	ISC_TIME,
	ISC_SIZE,
	ISC_DESC
} INDEX_SORT_COL;

typedef enum {
	ISO_ASC,
	ISO_DESC
} INDEX_SORT_ORD;

int directory_index(int fd,
		    DIRCONFIG const *conf, char const *uri, char const *path,
		    INDEX_SORT_COL col, INDEX_SORT_ORD ord);

INDEX_SORT_COL index_sort_col_from_arg(int c);
int index_sort_col_to_arg(INDEX_SORT_COL c);
INDEX_SORT_ORD index_sort_ord_from_arg(int c);
int index_sort_ord_to_arg(INDEX_SORT_ORD c);

int filename_is_special(char const *name);
int filename_is_hidden(char const *name, DIRCONFIG const *conf);
int filename_is_valid(char const *name);

void parse_template_string(char const *str);
void parse_template_file(char const *file_name);
